-------------------------------------------------------------------------------
--|> Config
-- Configure settings and options
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
-------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local AceConfig = LibStub("AceConfig-3.0")

-------------------------------------------------------------------------------
--|> LOCALS
-------------------------------------------------------------------------------
local C_Timer_After = C_Timer.After

-------------------------------------------------------------------------------
--|> SLASH COMMANDS
-------------------------------------------------------------------------------
--=> Comands Option Table
-------------------------------------------------------------------------------
weizPVP.SlashCommands = {
	name = "Slash Commands",
	order = -3,
	type = "group",
	args = {
		intro = {
			name = "weizPVP Slash Commands",
			type = "description",
			order = 1,
			cmdHidden = true
		},
		config = {
			name = "config",
			desc = "Opens the configuration menu.",
			type = "execute",
			func = function()
				InterfaceOptionsFrame_OpenToCategory("|cffeaeaeaweiz|r|cffffa012PVP|r")
				InterfaceOptionsFrame_OpenToCategory("|cffeaeaeaweiz|r|cffffa012PVP|r")
			end,
			guiHidden = true,
			order = 2
		},
		show = {
			name = "Show",
			desc = "Show the weizPVP window.",
			type = "execute",
			order = 3,
			func = function()
				if not InCombatLockdown() then
					weizPVP.HeaderFrame:Show()
					weizPVP.CoreFrame:Show()
				end
			end,
			dialogHidden = true
		},
		hide = {
			name = "Hide",
			desc = "Hide the weizPVP window.",
			type = "execute",
			order = 4,
			func = function()
				if not InCombatLockdown() then
					weizPVP.HeaderFrame:Hide()
					weizPVP.CoreFrame:Hide()
				end
			end,
			dialogHidden = true
		},
		lock = {
			name = "Lock",
			desc = "Toggle locking of the window's position and size.",
			type = "execute",
			order = 5,
			func = function()
				if not InCombatLockdown() then
					weizPVP:ToggleLocked(weizPVP.Options.WindowLocked or false)
				else
					--> In Combat
					weizPVP:PrintAddonMessage("Window can only be locked out of combat.")
				end
				weizPVP:SaveAddonData()
			end,
			dialogHidden = true
		},
		pin = {
			name = "Pin",
			desc = "Toggle pinning of the window.",
			type = "execute",
			order = 6,
			func = function()
				if weizPVP.CoreFrame.isLocked then
					weizPVP:PrintAddonMessage("Window is currently |cffff3838LOCKED|r. Can not pin until unlocked.")
				else
					weizPVP:TogglePinned()
				end
				weizPVP:SaveAddonData()
			end,
			dialogHidden = true
		},
		-------------------------------------------------------------------------------
		--!! HIDDEN COMMANDS ==========================================================
		--> DEBUG: Detects and manages ALL player characters; not just enemies.
		debug = {
			name = "DEBUG",
			desc = "Toggles DEBUG mode where ALL players of any factions are accepted as valid.",
			type = "execute",
			order = 60,
			func = function()
				if weizPVP.Options.GLOBAL_DEBUG == true then
					weizPVP.Options.GLOBAL_DEBUG = false
					weizPVP:PrintAddonMessage("|cff37ff37DEBUG DISABLED|r.")
				else
					weizPVP.Options.GLOBAL_DEBUG = true
					weizPVP:PrintAddonMessage("|cffff3838DEBUG ENABLED|r.")
				end
				weizPVP:SaveAddonData()
			end,
			cmdHidden = true
		},
		--> RESETALL: Wipes EVERYTHING; PlayerDB, Options, Current Variables, and then reloads the UI where default settings are applied.
		resetall = {
			name = "Reset All",
			desc = "Resets all settings and wipes player data",
			type = "execute",
			order = 61,
			func = function()
				weizPVP:ResetAll()
			end,
			cmdHidden = true
		}
		--!!===========================================================================
		-------------------------------------------------------------------------------
	}
}

--=> Register Slash Commands
-------------------------------------------------------------------------------
AceConfig:RegisterOptionsTable("weizPVP.SlashCommands", weizPVP.SlashCommands, {"wpvp", "weizpvp"})

-------------------------------------------------------------------------------
--|> INTERFACE OPTIONS: WOW GUI MENU
-------------------------------------------------------------------------------
--=> Options Table
-------------------------------------------------------------------------------
weizPVP.OptionsTable = {
	name = "|cffeaeaeaweiz|r|cffffa012PVP|r : Version |cff42dcf4" .. weizPVP.Version .. "|r",
	type = "group",
	args = {
		--> General Options Group
		-------------------------------------------
		general = {
			name = "General",
			type = "group",
			order = 1,
			args = {
				introGeneral = {
					name = "|cffffa012General Options|r\n",
					type = "description",
					fontSize = "large",
					order = 1
				},
				--> Options
				-------------------------------------------
				lock = {
					name = "Lock Window\n",
					desc = "Lock the window's location and size.",
					type = "toggle",
					order = 2,
					get = function(info)
						return weizPVP.Options.WindowLocked or false
					end,
					set = function(info, value)
						weizPVP:ToggleLocked(value)
					end
				},
				EnableBGs = {
					name = "Enabled in Battlegrounds\n",
					desc = "Enable weizPVP in Battlegrounds.",
					type = "toggle",
					order = 4,
					get = function(info)
						return weizPVP.Options.EnabledInBattlegrounds
					end,
					set = function(info, value)
						--> Double check that we were already disabled. If so, initialize addon again.
						weizPVP.Options.EnabledInBattlegrounds = value
						weizPVP.GetPVPZone()
					end
				}
			}
		},
		alerts = {
			name = "Alerts",
			type = "group",
			order = 2,
			args = {
				introAlerts = {
					name = "|cffffa012Alerts and Notifications|r\n",
					type = "description",
					fontSize = "large",
					order = 1
				},
				--> Options
				-------------------------------------------
				AlertDetectedPlayer = {
					name = "Play Sound: New Player \n",
					desc = "Play's an alert when a new player is added to the enemy list. (Will not spam if multiple detected at once.)",
					type = "toggle",
					order = 2,
					get = function(info)
						return weizPVP.Options.AudioAlerts.DetectedPlayerSound
					end,
					set = function(info, value)
						weizPVP.Options.AudioAlerts.DetectedPlayerSound = value
					end
				}
			}
		},
		crosshair = {
			name = "Crosshair",
			type = "group",
			order = 3,
			args = {
				introAlerts = {
					name = "|cffffa012Crosshair Options|r\n",
					type = "description",
					fontSize = "large",
					order = 1
				},
				--> Options
				-------------------------------------------
				Enabled = {
					name = "Enable Crosshairs\n",
					desc = "Toggles the Crosshairs on/off",
					type = "toggle",
					order = 2,
					get = function(info)
						return weizPVP.Options.Crosshair.Enabled
					end,
					set = function(info, value)
						if value == true then
							weizPVP:EnableCrosshair()
							weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
						else
							weizPVP:DisableCrosshair()
						end
					end
				},
				NameText = {
					name = "Enable Name Text \n",
					desc = "Shows your target's name on Crosshair overlay.",
					type = "toggle",
					order = 3,
					get = function(info)
						return weizPVP.Options.Crosshair.NameEnabled
					end,
					set = function(info, value)
						weizPVP.Options.Crosshair.NameEnabled = value
						if weizPVP.Options.Crosshair.Enabled then
							weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
						end
					end
				},
				Guildext = {
					name = "Enable Guild Text \n",
					desc = "Shows your target's guild name on Crosshair overlay.",
					type = "toggle",
					order = 4,
					get = function(info)
						return weizPVP.Options.Crosshair.GuildEnabled
					end,
					set = function(info, value)
						weizPVP.Options.Crosshair.GuildEnabled = value
						if weizPVP.Options.Crosshair.Enabled then
							weizPVP.CrosshairFrame:PLAYER_TARGET_CHANGED()
						end
					end
				}
			}
		}
	}
}

LibStub("AceConfig-3.0"):RegisterOptionsTable("weizPVP.OptionsTable", weizPVP.OptionsTable)
LibStub("AceConfigDialog-3.0"):AddToBlizOptions("weizPVP.OptionsTable", "|cffeaeaeaweiz|r|cffffa012PVP|r")

-------------------------------------------------------------------------------
--|> APPLY OPTIONS
-------------------------------------------------------------------------------
function weizPVP:ApplySavedOptions()
	--> Set window location and dimentions
	weizPVP:SetCoreFramePosition(weizPVP.Options.Frames.X, weizPVP.Options.Frames.Y, weizPVP.Options.Frames.Width, weizPVP.Options.Frames.Height)
	weizPVP:TogglePinned(weizPVP.Options.WindowPinned)
end
