-------------------------------------------------------------------------------
--|> Events
-- Manages all events
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local UnitIsPVPSanctuary = UnitIsPVPSanctuary
local UnitIsEnemy = UnitIsEnemy
local GetTime = GetTime
local UnitIsPlayer = UnitIsPlayer
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitHealthMax = UnitHealthMax
local UnitLevel = UnitLevel
local IsInInstance = IsInInstance
local bit_band = bit.band
local C_Timer_After = C_Timer.After
local InCombatLockdown = InCombatLockdown

-------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local LibCLHealth = LibStub("LibCombatLogHealth-1.0")

-------------------------------------------------------------------------------
--|> FUNCTIONS
-------------------------------------------------------------------------------

--=> Static Role Assignment
---------------------------------------------------------------------
function weizPVP:RoleAssign(class)
	local role = nil
	if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
		role = "DAMAGER"
	end
	return role
end

--=> Get Spec from Spell
-----------------------------------------------------------------------
function weizPVP:GetSpecFromSpell(SpellID)
	local specializationID = weizPVP.SpecDB[SpellID]
	return specializationID
end

--=> Get Role from Spec
-----------------------------------------------------------------------
function weizPVP:GetSpecRole(specializationID)
	local _, _, _, _, role, _, _ = GetSpecializationInfoByID(specializationID)
	return role
end

--=> Check for Valid Instance
-----------------------------------------------------------------------
function weizPVP:CheckValidInstance()
	--> Checking for Valid BG
	local inInstance = select(2, IsInInstance()) --> Get instance info
	local inBG = false
	if inInstance == "pvp" then
		inBG = true
	end

	if inBG == true and weizPVP.Options.EnabledInBattlegrounds == true then
		return true
	else
		return false
	end
end

--=> Check for Valid World Zone
-----------------------------------------------------------------------
function weizPVP:CheckValidWorldZone()
	--> Checking for Valid WPVP Zone
	local inInstance = select(2, IsInInstance())
	local WarModeEnabled = UnitIsWarModeDesired("player")
	if inInstance == "none" and WarModeEnabled == true then
		return true
	else
		return false
	end
end

--=> Get PVP Zone
-----------------------------------------------------------------------
local ZoneTimerStartTime = GetTime()
local ValidBG
local ValidWorldZone
function weizPVP.GetPVPZone()
	if ZoneTimerStartTime + 1 <= GetTime() then
		ZoneTimerStartTime = GetTime()
		if GetZoneText() == "" then
			C_Timer_After(0.5, weizPVP.GetPVPZone)
			return
		end
		ValidWorldZone = weizPVP:CheckValidWorldZone()
		ValidBG = weizPVP.CheckValidInstance()
		if not InCombatLockdown() then
			if ValidWorldZone or ValidBG then
				weizPVP.OnEnable()
			else
				weizPVP.OnDisable()
			end
		end
	else
		return
	end
end

-------------------------------------------------------------------------------
--|> EVENTS
-------------------------------------------------------------------------------

--=> EVENT: Entering Battleground Instance
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringBattlegroundEvent()
	C_Timer_After(1, weizPVP.GetPVPZone)
	wipe(weizPVP.CurrentNameplates)
end

--=> EVENT: ZoneChangedNewAreaEvent
-----------------------------------------------------------------------
function weizPVP:ZoneChangedNewAreaEvent()
	C_Timer_After(1, weizPVP.GetPVPZone)
end
--=> EVENT: Player Entering World
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringWorldEvent()
	wipe(weizPVP.CurrentNameplates)
	C_Timer_After(1, weizPVP.GetPVPZone)
end

--=> EVENT TIMER; ONE SECOND REPEATING
-----------------------------------------------------------------------
function weizPVP:OneSecondTimer()
	--> FPS Update
	weizPVP.Display.CurrentFPS = GetFramerate()

	--> Manage list timeouts
	weizPVP.ManageListTimeouts()
end

--=> EVENT: COMBAT_LOG_UNFILTERED
-----------------------------------------------------------------------
local CLogTime
local newDataFound
local playerData
local stealth
local dead
local health
local role
local sanctuary
local specID
local _, event, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, arg12, arg13
function weizPVP:CombatLogEvent()
	_, --timestamp
		event,
		_, --hideCaster
		srcGUID,
		srcName,
		srcFlags,
		_, --sourceRaidFlags
		dstGUID,
		dstName,
		dstFlags,
		_, --destRaidFlags
		arg12, --spellID
		arg13, --spellName
		_,
		_,
		_ = CombatLogGetCurrentEventInfo()

	CLogTime = GetTime()

	--> Dont scan yourself when you are both the source and destination
	if (dstName == srcName and srcName == weizPVP.Player.Name) and weizPVP.Options.GLOBAL_DEBUG == false then
		return
	end

	--> Update time only
	if weizPVP.PlayerActiveCache[srcName] and weizPVP.PlayerActiveCache[srcName].Estimated == false and event ~= "UNIT_DIED" then
		if weizPVP.NearbyList[srcName].TimeUpdated + 3 < CLogTime then
			weizPVP:AddPlayerDataToNearby(srcName, false)
		end
		if weizPVP.PlayerActiveCache[dstName] and weizPVP.PlayerActiveCache[dstName].Estimated == false then
			if weizPVP.NearbyList[dstName].TimeUpdated + 3 < CLogTime then
				if weizPVP.PlayerActiveCache[dstName] and weizPVP.PlayerActiveCache[dstName].Estimated == false then
					weizPVP:AddPlayerDataToNearby(dstName, false)
				end
			end
		end
		return
	end

	local OBJECT_TYPE_PLAYER = 0x00000400
	local OBJECT_TYPE_ENEMY = 0x00000040

	--> Source Unit Info
	-------------------------------------------------------------------------------------------------------------------
	if (bit_band(srcFlags, OBJECT_TYPE_ENEMY) ~= 0 or weizPVP.Options.GLOBAL_DEBUG) and bit_band(srcFlags, OBJECT_TYPE_PLAYER) ~= 0 and srcGUID and srcName and srcName ~= "Unknown" then
		newDataFound = false
		role = nil
		stealth = nil
		dead = false

		--> If Player doesn't exist in DB
		if not weizPVP.PlayerDB[srcName] then
			newDataFound, role = weizPVP:GetLevelData(true, event, srcName, srcGUID, arg12, arg13)
		elseif weizPVP.PlayerDB[srcName] and weizPVP.PlayerDB[srcName].Estimated == true then
			--> Player exists, but data is estimated; try to get level based on ability used
			newDataFound, role = weizPVP:GetLevelData(true, event, srcName, srcGUID, arg12, arg13)
		end

		--> Check for spec and role based off spellID
		specID = weizPVP:GetSpecFromSpell(arg12)
		if specID ~= nil then
			role = weizPVP:GetSpecRole(specID) --> "DAMAGER", "TANK", or "HEALER"
			if role then
				weizPVP:UpdatePlayerDB(srcName, nil, nil, nil, nil, true, false, role)
			end
		end

		--> Check for Stealth
		if event == "SPELL_AURA_APPLIED" and ((arg13 == "Stealth") or (arg13 == "Prowl") or (arg13 == "Invisibility")) then
			stealth = true
			dead = false
			--> Stealth Alert
			sanctuary = UnitIsPVPSanctuary("player")
			if not sanctuary or weizPVP.Options.GLOBAL_DEBUG then --> Show alert only if not in sanctuary for Stealth players
				if arg13 ~= "Invisibility" then --> Stealth and Prowl
					weizPVP:ShowStealthAlert("Stealth Detected!", srcName, {["r"] = 0, ["b"] = 0, ["g"] = 0, ["a"] = 0.8})
				else
					weizPVP:ShowStealthAlert("Invisability Detected!", srcName, {["r"] = 0, ["b"] = 0, ["g"] = 0, ["a"] = 0.8})
				end
				weizPVP:UpdatePlayerActiveCache(srcName, newDataFound, nil, true, false, role)
			end
		end

		--> No new data was added or updated, send the few things we do know can update(stealth/dead)
		if newDataFound ~= true then
			weizPVP:UpdatePlayerDB(srcName, nil, nil, nil, nil, true, false)
		--> weizPVP:UpdatePlayerDB(name, class, level, race, guild, estimated, dataUpdated)
		end

		--> UPDATE
		weizPVP:UpdatePlayerActiveCache(srcName, newDataFound, nil, stealth, dead, role)
	end

	--> Destination Unit
	-------------------------------------------------------------------------------------------------------------------
	if (bit_band(dstFlags, OBJECT_TYPE_ENEMY) ~= 0 or weizPVP.Options.GLOBAL_DEBUG) and bit_band(dstFlags, OBJECT_TYPE_PLAYER) ~= 0 and dstGUID and dstName and dstName ~= "Unknown" then
		newDataFound = false
		playerData = weizPVP.PlayerDB[dstName]
		stealth = nil
		dead = false
		health = nil
		if not playerData or playerData.Estimated then
			newDataFound = weizPVP:GetLevelData(false, event, dstName, dstGUID, arg12, arg13)
		end
		weizPVP:UpdatePlayerDB(dstName, nil, nil, nil, nil, nil, newDataFound)
		--> weizPVP:UpdatePlayerDB(name, class, level, race, guild, estimated, dataUpdated)

		--> if unit died
		if event == "UNIT_DIED" then
			dead = true
			health = 0
			stealth = false
		end
		weizPVP:UpdatePlayerActiveCache(dstName, newDataFound, health, stealth, dead, nil)
	end
end

--=> EVENT: LibCLHealth (Lib to get health from combat log)
-----------------------------------------------------------------------
local info = {}
local healthPercent
if LibCLHealth then
	LibCLHealth.RegisterCallback(
		weizPVP,
		"COMBAT_LOG_HEALTH",
		function(event, unit, eventType)
			if (UnitIsEnemy("player", unit) or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer(unit) then
				healthPercent = tonumber(LibCLHealth.UnitHealth(unit) / UnitHealthMax(unit) * 100)
				info = {}
				info.Name = GetUnitName(unit, true)
				if info.Name ~= "Unknown" then
					info.Dead = UnitIsDeadOrGhost(unit)
					info.Stealth = nil
					info.Health = healthPercent
					weizPVP:UpdatePlayerActiveCache(info.Name, true, info.Health, info.Stealth, info.Dead, nil)
					return true
				end
			end
		end
	)
end

--=> EVENT: Player Mouseover
-----------------------------------------------------------------------
function weizPVP:PlayerMouseoverEvent()
	weizPVP:GetUnitData("mouseover")
end

--=> EVENT: Player Target Changed
-- -----------------------------------------------------------------------
function weizPVP:PlayerTargetEvent()
	weizPVP:GetUnitData("target")
	weizPVP:ChangeTargetIcon()
end

--=> EVENT: NAMEPLATE_ADDED
-- -----------------------------------------------------------------------
local name
function weizPVP:NameplateAdded(_, unit)
	if (UnitIsEnemy("player", unit) or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer(unit) then
		name = GetUnitName(unit, true)
		--> Update time only
		if weizPVP.PlayerActiveCache[name] and weizPVP.PlayerActiveCache[name].Estimated == false then
			weizPVP:AddPlayerDataToNearby(name, false)
		else
			weizPVP:GetUnitData(unit)
		end
		weizPVP.CurrentNameplates[name] = true
	end
end

--=> EVENT: NAMEPLATE REMOVED
-- -----------------------------------------------------------------------
function weizPVP:NameplateRemoved(_, unit)
	name = GetUnitName(unit, true)
	if weizPVP.PlayerActiveCache[name] then
		weizPVP:AddPlayerDataToNearby(name, false)
	end
	weizPVP.CurrentNameplates[name] = nil
end

--=> EVENT: ENTERING COMBAT
-----------------------------------------------------------------------
function weizPVP:EnteringCombat()
	weizPVP:SetStatusBarMessage("|cFFF4564D +++ IN COMBAT|r +", false, 0)
end

--=> EVENT: LEAVING COMBAT
-----------------------------------------------------------------------
function weizPVP:LeavingCombat()
	weizPVP:SetStatusBarMessage("|cFF53F493 --- OUT OF COMBAT|r -", true, 3)
	weizPVP:ManageListTimeouts()
end

--=> EVENT: Player Leveled Up
-----------------------------------------------------------------------
function weizPVP:PlayerLevelUp()
	weizPVP.Player.Level = UnitLevel("player") --> Update player level
end
